@@ -24,6 +24,22 @@ logger = logging.getLogger(__name__)
 
 
 class EVA:
+    """
+    Extreme Value Analysis (EVA) class.
+
+    This class brings together most of the tools available in the pyextremes package
+    bundled together in a pipeline to perform univariate extreme value analysis.
+
+    A typical workflow using the EVA class would consist of the following:
+        - extract extreme values (.get_extremes)
+        - fit a model (.fit_model)
+        - generate outputs (.get_summary)
+        - visualize the model (.plot_diagnostic, .plot_return_values)
+
+    Multiple additional graphical and numerical methods are available
+    within this class to analyze extracted extreme values, visualize them,
+    assess goodness-of-fit of selected model, and to visualize its outputs.
+    """
 
     __slots__ = [
         "__data",
@@ -37,20 +53,7 @@ class EVA:
 
     def __init__(self, data: pd.Series) -> None:
         """
-        Extreme Value Analysis (EVA) class.
-
-        This class brings together most of the tools available in the pyextremes package
-        bundled together in a pipeline to perform univariate extreme value analysis.
-
-        A typical workflow using the EVA class would consist of the following:
-            - extract extreme values (.get_extremes)
-            - fit a model (.fit_model)
-            - generate outputs (.get_summary)
-            - visualize the model (.plot_diagnostic, .plot_return_values)
-
-        Multiple additional graphical and numerical methods are available
-        within this class to analyze extracted extreme values, visualize them,
-        assess goodness-of-fit of selected model, and to visualize its outputs.
+        Initialize EVA model.
 
         Parameters
         ----------
@@ -59,55 +62,56 @@ class EVA:
             Index must be date-time and values must be numeric.
 
         """
-        # Ensure that 'data' is pandas Series
+        # Ensure that `data` is pandas Series
         if not isinstance(data, pd.Series):
             raise TypeError(
-                f"invalid type in {type(data)} for the 'data' argument, "
+                f"invalid type in '{type(data).__name__}' for the `data` argument, "
                 f"must be pandas.Series"
             )
 
-        # Copy 'data' to ensure the original Series object it is not mutated
+        # Copy `data` to ensure the original Series object it is not mutated
         data = data.copy(deep=True)
 
-        # Ensure that 'data' has correct index and value dtypes
+        # Ensure that `data` has correct index and value dtypes
         if not np.issubdtype(data.dtype, np.number):
             try:
-                data = data.astype(np.float64)
-                message = "'data' values are not numeric - converted to numeric"
-                logger.info(message)
+                message = "`data` values are not numeric - converting to numeric"
+                logger.debug(message)
                 warnings.warn(message=message, category=RuntimeWarning)
+                data = data.astype(np.float64)
             except ValueError as _error:
                 raise TypeError(
-                    f"invalid dtype in {data.dtype} for the 'data' argument, "
+                    f"invalid dtype in {data.dtype} for the `data` argument, "
                     f"must be numeric (subdtype of numpy.number)"
                 ) from _error
         if not isinstance(data.index, pd.DatetimeIndex):
             raise TypeError(
-                f"index of 'data' must be a sequence of date-time objects, "
+                f"index of `data` must be a sequence of date-time objects, "
                 f"not {data.index.inferred_type}"
             )
 
-        # Ensure that 'data' is sorted
+        # Ensure that `data` is sorted
         if not data.index.is_monotonic_increasing:
             message = (
-                "'data' index is not sorted in ascending order - sorting data by index"
+                "`data` index is not sorted in ascending order - "
+                "sorting `data` by index"
             )
-            logger.info(message)
+            logger.debug(message)
             warnings.warn(message=message, category=RuntimeWarning)
             data = data.sort_index(ascending=True)
 
-        # Ensure that 'data' has no invalid entries
+        # Ensure that `data` has no invalid entries
         n_nans = data.isna().sum()
         if n_nans > 0:
             message = (
-                f"{n_nans:,d} Null values found in 'data' - removing invalid entries"
+                f"{n_nans:,d} Null values found in `data` - removing invalid entries"
             )
-            logger.info(message)
+            logger.debug(message)
             warnings.warn(message=message, category=RuntimeWarning)
             data = data.dropna()
 
-        # Set the 'data' attribute
-        self.__data = data
+        # Set the `data` attribute
+        self.__data: pd.Series = data
 
         # Initialize attributes related to extreme value extraction
         self.__extremes: typing.Optional[pd.Series] = None
@@ -119,16 +123,13 @@ class EVA:
         # Initialize attributes related to model fitting
         self.__model: typing.Optional[typing.Union[MLE, Emcee]] = None
 
-        logger.info(
-            "successfully initialized EVA object with data of length %d"
-            % len(self.data)
-        )
+        logger.info("successfully initialized EVA object")
 
     @property
     def data(self) -> pd.Series:
         return self.__data
 
-    def __get_extremes_attribute(self, attribute_name: str) -> typing.Any:
+    def __get_extremes_attribute(self, attribute_name: str):
         value = getattr(self, f"_EVA__{attribute_name}")
         if value is None:
             raise AttributeError(
@@ -151,35 +152,40 @@ class EVA:
         return self.__get_extremes_attribute("extremes_type")
 
     @property
-    def extremes_kwargs(self) -> typing.Dict[str, typing.Any]:
+    def extremes_kwargs(self) -> dict:
         return self.__get_extremes_attribute("extremes_kwargs")
 
     @property
     def extremes_transformer(self) -> ExtremesTransformer:
         return self.__get_extremes_attribute("extremes_transformer")
 
-    @property
-    def model(self) -> typing.Union[MLE, Emcee]:
+    def __get_model_attribute(self, attribute_name: typing.Optional[str] = None):
         if self.__model is None:
             raise AttributeError(
                 "model must first be assigned using the '.fit_model' method"
             )
-        else:
+        if attribute_name is None:
             return self.__model
+        else:
+            return getattr(self.__model, attribute_name)
+
+    @property
+    def model(self) -> typing.Union[MLE, Emcee]:
+        return self.__get_model_attribute()
 
     @property
     def distribution(self) -> Distribution:
-        return self.model.distribution
+        return self.__get_model_attribute("distribution")
 
     @property
     def loglikelihood(self) -> float:
-        return self.model.loglikelihood
+        return self.__get_model_attribute("loglikelihood")
 
     @property
     def AIC(self) -> float:
-        return self.model.AIC
+        return self.__get_model_attribute("AIC")
 
-    def __repr__(self) -> str:  # pragma: no cover
+    def __repr__(self) -> str:
         # Width of repr block
         width = 88
 
@@ -439,11 +445,11 @@ class EVA:
                     coerce - get extreme values for blocks with no data
                         as mean of all other extreme events in the series
                         with index being the middle point of corresponding interval
-                    min_last_block : float, optional
-                        Minimum data availability ratio (0 to 1) in the last block
-                        for it to be used to extract extreme value from.
-                        This is used to discard last block when it is too short.
-                        If None (default), last block is always used.
+                min_last_block : float, optional
+                    Minimum data availability ratio (0 to 1) in the last block
+                    for it to be used to extract extreme value from.
+                    This is used to discard last block when it is too short.
+                    If None (default), last block is always used.
             if method is POT:
                 threshold : float
                     Threshold used to find exceedances.
@@ -453,13 +459,16 @@ class EVA:
 
         """
         message = f"for method='{method}' and extremes_type='{extremes_type}'"
-        logger.debug("extracting extreme values %s" % message)
+        logger.debug("extracting extreme values %s", message)
         self.__extremes = get_extremes(
-            method=method, ts=self.data, extremes_type=extremes_type, **kwargs
+            method=method,
+            ts=self.data,
+            extremes_type=extremes_type,
+            **kwargs,
         )
         self.__extremes_method = method
         self.__extremes_type = extremes_type
-        logger.info("successfully extracted extreme values %s" % message)
+        logger.info("successfully extracted extreme values %s", message)
 
         logger.debug("collecting extreme value properties")
         self.__extremes_kwargs = {}
@@ -492,6 +501,191 @@ class EVA:
         logger.info("removing any previously declared models")
         self.__model = None
 
+    def set_extremes(self, extremes: pd.Series, **kwargs) -> None:
+        """
+        Set extreme values.
+
+        This method is used to set extreme values onto the model instead
+        of deriving them from data directly using the 'get_extremes' method.
+        This way user can set extremes calculated using a custom methodology.
+
+        Parameters
+        ----------
+        extremes : pd.Series
+            Time series of extreme values to be set onto the model.
+            Must be numeric, have date-time index, and have the same name
+            as self.data.
+        kwargs:
+            method : str, optional
+                Extreme value extraction method.
+                Supported values:
+                    BM (default) - Block Maxima
+                    POT - Peaks Over Threshold
+            extremes_type : str, optional
+                high (default) - extreme high values
+                low - extreme low values
+            if method is BM:
+                block_size : str or pandas.Timedelta, optional
+                    Block size.
+                    By default is calculated as average distance between `extremes`.
+                errors : str, optional
+                    raise - raise an exception
+                        when encountering a block with no data
+                    ignore (default) - ignore blocks with no data
+                    coerce - get extreme values for blocks with no data
+                        as mean of all other extreme events in the series
+                        with index being the middle point of corresponding interval
+                min_last_block : float, optional
+                    Minimum data availability ratio (0 to 1) in the last block
+                    for it to be used to extract extreme value from.
+                    This is used to discard last block when it is too short.
+                    If None (default), last block is always used.
+            if method is POT:
+                threshold : float, optional
+                    Threshold used to find exceedances.
+                    By default is taken as smallest value.
+                r : str or pandas.Timedelta, optional
+                    Duration of window used to decluster the exceedances.
+                    By default r='24H' (24 hours).
+
+        """
+        # Validate `extremes`
+        if not isinstance(extremes, pd.Series):
+            raise TypeError(
+                f"invalid type in '{type(extremes).__name__}' for the `extremes` "
+                f"argument, must be pandas.Series"
+            )
+        extremes = extremes.copy(deep=True)
+        if not isinstance(extremes.index, pd.DatetimeIndex):
+            raise TypeError("invalid index type for `extremes`, must be date-time")
+        if not np.issubdtype(extremes.dtype, np.number):
+            raise TypeError("`extremes` must have numeric values")
+        if extremes.name is None:
+            extremes.name = self.data.name
+        else:
+            if extremes.name != self.data.name:
+                raise ValueError("`extremes` name doesn't match that of `data`")
+        if not (
+            (extremes.index.min() >= self.data.index.min())
+            and (extremes.index.max() <= self.data.index.max())
+        ):
+            raise ValueError("`extremes` time range must fit within that of data")
+
+        # Get `method`
+        method: str = kwargs.pop("method", "BM")
+        if method not in ["BM", "POT"]:
+            raise ValueError(f"`method` must be either 'BM' or 'POT', not '{method}'")
+
+        # Get `extremes_type`
+        extremes_type: str = kwargs.pop("extremes_type", "high")
+        if extremes_type not in ["high", "low"]:
+            raise ValueError(
+                f"`extremes_type` must be either 'BM' or 'POT', not '{extremes_type}'"
+            )
+
+        # Get `extremes_kwargs`
+        extremes_kwargs = {}
+        if method == "BM":
+            # Get `block_size`
+            extremes_kwargs["block_size"] = pd.to_timedelta(
+                kwargs.pop(
+                    "block_size",
+                    (extremes.index.max() - extremes.index.min()) / (len(extremes) - 1),
+                )
+            )
+            if extremes_kwargs["block_size"] <= pd.to_timedelta("0D"):
+                raise ValueError(
+                    "`block_size` must be a positive timedelta, not %s"
+                    % extremes_kwargs["block_size"]
+                )
+
+            # Get `errors`
+            extremes_kwargs["errors"] = kwargs.pop("errors", "ignore")
+            if extremes_kwargs["errors"] not in ["raise", "ignore", "coerce"]:
+                raise ValueError(
+                    f"invalid value in '{extremes_kwargs['errors']}' "
+                    f"for the `errors` argument"
+                )
+
+            # Get `min_last_block`
+            extremes_kwargs["min_last_block"] = kwargs.pop("min_last_block", None)
+            if extremes_kwargs["min_last_block"] is not None:
+                if not 0 <= extremes_kwargs["min_last_block"] <= 1:
+                    raise ValueError(
+                        "`min_last_block` must be a number in the [0, 1] range"
+                    )
+
+        elif method == "POT":
+            # Get `threshold`
+            extremes_kwargs["threshold"] = kwargs.pop(
+                "threshold",
+                {
+                    "high": extremes.min(),
+                    "low": extremes.max(),
+                }[extremes_type],
+            )
+            if (
+                extremes_type == "high"
+                and extremes_kwargs["threshold"] > extremes.values.min()
+            ) or (
+                extremes_type == "low"
+                and extremes_kwargs["threshold"] < extremes.values.max()
+            ):
+                raise ValueError("invalid `threshold` value")
+
+            # Get `r`
+            extremes_kwargs["r"] = pd.to_timedelta(kwargs.pop("r", "24H"))
+            if extremes_kwargs["r"] <= pd.to_timedelta("0D"):
+                raise ValueError(
+                    "`r` must be a positive timedelta, not %s" % extremes_kwargs["r"]
+                )
+
+        else:
+            raise RuntimeError
+
+        # Check for unrecognized kwargs
+        if len(kwargs) != 0:
+            raise TypeError(
+                f"unrecognized arguments passed in: {', '.join(kwargs.keys())}"
+            )
+
+        # Set attributes
+        self.__extremes = extremes
+        self.__extremes_method = method
+        self.__extremes_type = extremes_type
+        self.__extremes_kwargs = extremes_kwargs
+        self.__extremes_transformer = ExtremesTransformer(
+            extremes=self.__extremes,
+            extremes_type=self.__extremes_type,
+        )
+        self.__model = None
+        logger.info("successfully set extremes")
+
+    @classmethod
+    def from_extremes(cls, extremes: pd.Series, **kwargs):
+        """
+        Create an EVA model using pre-defined `extremes`.
+
+        A typical reason to use this method is when full timeseries is not available
+        and only the extracted extremes (i.e. annual maxima) are known.
+
+        Parameters
+        ----------
+        extremes : pd.Series
+            Time series of extreme values.
+        kwargs:
+            See '.set_extremes' method documentation.
+
+        Returns
+        -------
+        EVA
+            EVA model initialized with `extremes`.
+
+        """
+        model = cls(data=extremes)
+        model.set_extremes(extremes=extremes, **kwargs)
+        return model
+
     def plot_extremes(
         self,
         figsize: tuple = (8, 5),
@@ -590,7 +784,8 @@ class EVA:
         if distribution is None:
             logger.debug(
                 "selecting default distribution for extremes extracted using the "
-                "'%s' method" % self.extremes_method
+                "'%s' method",
+                self.extremes_method,
             )
 
             # Prepare list of candidate distributions
@@ -622,7 +817,9 @@ class EVA:
                     distribution = distribution_name
                     aic = new_aic
             logger.info(
-                "selected '%s' distribution with AIC score %s" % (distribution, aic)
+                "selected '%s' distribution with AIC score %s",
+                distribution,
+                aic,
             )
 
         # Get distribution name
@@ -684,8 +881,9 @@ class EVA:
                 )
             }
             logger.debug(
-                "freezing location parameter (floc) at %s for '%s' distribution"
-                % (distribution_kwargs["floc"], distribution_name)
+                "freezing location parameter (floc) at %s for '%s' distribution",
+                distribution_kwargs["floc"],
+                distribution_name,
             )
 
         # Fit model to transformed extremes
