@@ -10,12 +10,14 @@ if __name__ == "__main__":
     data = data.loc[data.index.year >= 1925]
     data = (
         data
-        - (data.index.array - pd.to_datetime("1992")) / pd.to_timedelta("1Y") * 2.87e-3
+        - (data.index.array - pd.to_datetime("1992"))
+        / pd.to_timedelta("365.2425D")
+        * 2.87e-3
     )
     for method in ["BM", "POT"]:
         for extremes_type in ["high", "low"]:
             if method == "BM":
-                kwargs = {"block_size": "1Y", "errors": "ignore"}
+                kwargs = {"block_size": "365.2425D", "errors": "ignore"}
             else:
                 if extremes_type == "high":
                     kwargs = {"threshold": 1.35, "r": "24H"}
